#!/usr/bin/python

if __name__ == "__main__" :
    
    import sys
    import filter, calibcant_vib_analyze
    from optparse import OptionParser

    usage_string = ('%prog <input-file>\n'
                    '2008, W. Trevor King.\n'
                    '\n'
                    'Apply various filters to data y(x).'
                    '<input-file> should be whitespace-delimited, 2 column ASCII\n'
                    'without a header line.\n'
                    'e.g: "<x>\\t<y>\\n"')
    parser = OptionParser(usage=usage_string, version='%prog 0.1')
    parser.add_option('-o', '--output-file', dest='ofilename',
                      help='write output to FILE (default stdout)',
                      type='string', metavar='FILE')
    parser.add_option('-w', '--width', dest='width', default=None,
                      help='window width (i +/- width, inclusive)',
                      type='int', metavar='WIDTH')
    parser.add_option('-t', '--type', dest='type', default='mean',
                      help='filter type (default %default)',
                      type='string', metavar='TYPE')
    parser.add_option('-m', '--min-freq', dest='min_freq', default=500,
                      help='minimum frequency in Hz (default %default)',
                      type='float', metavar='FREQ')
    parser.add_option('-M', '--max-freq', dest='max_freq', default=7000,
                      help='maximum frequency in Hz (default %default)',
                      type='float', metavar='FREQ')
    parser.add_option('-g', '--gnuplot', dest='gnuplot', action='store_true',
                      help='Print gnuplot fit check script to stderr',
                      default=False)
    parser.add_option('-v', '--verbose', dest='verbose', action='store_true',
                      help='Print lots of debugging information',
                      default=False)

    options,args = parser.parse_args()
    parser.destroy()
    assert len(args) >= 1, "Need an input file"
        
    ifilename = args[0]

    # mirror filter behavior:
    freq,psd = filter.read_data(ifilename)

    if options.type == 'mean' :
        winfunc = filter.windowed_mean_point
    elif options.type == 'median' :
        winfunc = filter.windowed_median_point
    else :
        raise Exception, "unrecognized window type '%s'" % options.type

    psd = filter.windowed_filter(psd, winfunc, width=options.width)

    # mirror calibcant_vib_analyze behavior:
    (A,B,C) = calibcant_vib_analyze.fit_psd(freq, psd,
                                            minFreq=options.min_freq,
                                            maxFreq=options.max_freq)
    if options.verbose :
        print >> sys.stderr, "Fit f(x) = C / ((A**2-x**2)**2 + (x*B)**2) with"
        print >> sys.stderr, "A = %g, \t B = %g, \t C = %g" % (A, B, C)
    if options.gnuplot :
        print >> sys.stderr, \
            calibcant_vib_analyze.gnuplot_check_fit(ifilename, A, B, C)

    area = calibcant_vib_analyze.lorentzian_area(A,B,C)

    if options.ofilename != None :
        print >> file(options.ofilename, 'w'), area
    else :
        print area
